==> Rserve_0.1-9.tar.gz: Rserve/src/Rcli.c <==
/*
 *  Rcli : a simple client for Rserv basically to demostrate the functionality
 *  Copyright (C) 2002 Simon Urbanek
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: Rcli.c,v 1.6 2003/04/21 02:00:23 starsoft Exp $
 */

/* external defines:
   READLINE - if defined, GNU readline support is enabled
   SWAPEND  - needs to be defined for platforms with inverse endianess related to Intel
*/

#define SOCK_ERRORS
#define USE_SNPRINTF

#if defined __GNUC__ && !defined unix
#define unix
#endif

#include <stdio.h>
#include <sisocks.h>
#ifdef LSOCK
#include <sys/un.h>
#endif
#include "Rsrv.h"

#ifdef READLINE
#include <readline/readline.h>
#include <readline/history.h>
#endif

int port = default_Rsrv_port;
char *myID= "Rsrv0100QAP1";

void doCmd(int s, int cmd, int len, void *par) {
  struct phdr ph;
  int i,n,j;
  char buf[128];

  memset(&ph,0,sizeof(ph));
  ph.len=itop(len);
  ph.cmd=itop(cmd);
  send(s,&ph,sizeof(ph),0);
  if (len>0) {
    printf("attached data:\n");
    i=0;
    while(i<len) printf("%02x ",((char*)par)[i++]);
    puts("");
    send(s,par,len,0);
  }
  n=recv(s,&ph,sizeof(ph),0);
  if (n!=sizeof(ph)) {
    printf("wrong response or socked closed (n=%d)\n",n);
    closesocket(s);
    return;
  };
  i=ph.len=ptoi(ph.len);
  ph.cmd=ptoi(ph.cmd);
  printf("RESP: %08x (%d bytes attached)\n",ph.cmd,ph.len);
  if (i>0) {
    while(n=recv(s,buf,(i>128)?128:i,0)) {
      if (n>0) i-=n;
      j=0;
      while(j<n) printf("%02x ",(unsigned char)buf[j++]);
      if (i<1) break;
    };
    if (i>0) {
      printf("error while reading response parameters\n");
      closesocket(s);
      return;
    };
    printf("\n");
  };
};

int main(int argc, char **argv) {
  SOCKET s;
#ifdef LSOCK
  struct sockaddr_un sa;
#else
  SAIN sa;
#endif

  char IDstring[33];
  int n,i,j;
#ifdef READLINE
  char *c;
#else
  char c[1024];
#endif
  char *rq;

#ifdef LSOCK
#define FAMILY AF_LOCAL
  memset(&sa,0,sizeof(sa));
  sa.sun_family=AF_LOCAL;
  strcpy(sa.sun_path,(argc>1)?argv[1]:"/tmp/Rserve");
#else
#define FAMILY AF_INET
  build_sin(&sa,(argc>1)?argv[1]:"127.0.0.1",port);
#endif
  IDstring[32]=0;
  
  s=socket(FAMILY,SOCK_STREAM,0);
  i=connect(s,(SA*)&sa,sizeof(sa));
  if (i==-1) {
    printf("unable to connect.\n");
    closesocket(s);
    return 1;
  };
  printf("connected.\n");
  n=recv(s,IDstring,32,0);
  if (n!=32) {
    printf("no valid IDstring!\n");
    closesocket(s);
    return 1;
  };
  printf("ID string: %s\n",IDstring);
  if (strncmp(IDstring,myID,12)) {
    printf("Cannot communicate with that fellow. He's too good for me.\n");
    closesocket(s);
    return 1;
  };
  printf("Ok, sounds good.\n");
  do {
#ifdef READLINE
    c=readline("Rcli> ");
    if (c && *c) add_history(c);
#else
    printf("Rcli> ");
    c[1023]=0;
    fgets(c,1023,stdin);
    rq=c; while(*rq) { if (*rq=='\n') rq=0; rq++; };
#endif

    if (!strcmp(c,"q") || !strcmp(c,"quit") || !strcmp(c,"exit")) {
      printf("ok, you got enough, right? leaving.\n");
      break;
    };
    if (!strcmp(c,"shutdown")) {
      printf("performing shutdown.\n");
      doCmd(s,CMD_shutdown,0,0);
    } else if (*c) {
      rq=(char*)malloc(strlen(c)+16);
      *((int*)rq)=itop(SET_PAR(DT_STRING,strlen(c)+2));
      strcpy(rq+4,c);
      strcat(rq+4,"\n");
      printf("sending request\n");
      doCmd(s,CMD_eval,strlen(c)+6,rq);
      free(rq);
    }
  } while (1);

  closesocket(s);
  return 0;
};

==> Rserve_0.2-1.tar.gz: Rserve/src/Rcli.c <==
/*
 *  Rcli : a simple client for Rserv basically to demostrate the functionality
 *  Copyright (C) 2002 Simon Urbanek
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: Rcli.c,v 1.8 2003/08/16 16:44:50 starsoft Exp $
 */

/* external defines:
   READLINE - if defined, GNU readline support is enabled
   SWAPEND  - needs to be defined for platforms with inverse endianess related to Intel
*/

#define SOCK_ERRORS
#define USE_SNPRINTF

#if defined __GNUC__ && !defined unix
#define unix
#endif

#include <stdio.h>
#include <sisocks.h>
#ifdef LSOCK
#include <sys/un.h>
#endif
#include "Rsrv.h"

#ifdef READLINE
#include <readline/readline.h>
#include <readline/history.h>
#endif

int port = default_Rsrv_port;
char *myID= "Rsrv0100QAP1";

void doCmd(int s, int cmd, int len, void *par) {
  struct phdr ph;
  int i,n,j;
  char buf[128];

  memset(&ph,0,sizeof(ph));
  ph.len=itop(len);
  ph.cmd=itop(cmd);
  send(s,&ph,sizeof(ph),0);
  if (len>0) {
    printf("attached data:\n");
    i=0;
    while(i<len) printf("%02x ",((char*)par)[i++]);
    puts("");
    send(s,par,len,0);
  }
  n=recv(s,&ph,sizeof(ph),0);
  if (n!=sizeof(ph)) {
    printf("wrong response or socked closed (n=%d)\n",n);
    closesocket(s);
    return;
  };
  i=ph.len=ptoi(ph.len);
  ph.cmd=ptoi(ph.cmd);
  printf("RESP: %08x (%d bytes attached)\n",ph.cmd,ph.len);
  if (i>0) {
    while(n=recv(s,buf,(i>128)?128:i,0)) {
      if (n>0) i-=n;
      j=0;
      while(j<n) printf("%02x ",(unsigned char)buf[j++]);
      if (i<1) break;
    };
    if (i>0) {
      printf("error while reading response parameters\n");
      closesocket(s);
      return;
    };
    printf("\n");
  };
};

int main(int argc, char **argv) {
  SOCKET s;
#ifdef LSOCK
  struct sockaddr_un sa;
#else
  SAIN sa;
#endif

  char IDstring[33];
  int n,i,j;
#ifdef READLINE
  char *c;
#else
  char c[1024];
#endif
  char *rq;

#ifdef LSOCK
#define FAMILY AF_LOCAL
  memset(&sa,0,sizeof(sa));
  sa.sun_family=AF_LOCAL;
  strcpy(sa.sun_path,(argc>1)?argv[1]:"/tmp/Rserve");
#else
#define FAMILY AF_INET
  build_sin(&sa,(argc>1)?argv[1]:"127.0.0.1",port);
#endif
  IDstring[32]=0;
  
  s=socket(FAMILY,SOCK_STREAM,0);
  i=connect(s,(SA*)&sa,sizeof(sa));
  if (i==-1) {
    printf("unable to connect.\n");
    closesocket(s);
    return 1;
  };
  printf("connected.\n");
  n=recv(s,IDstring,32,0);
  if (n!=32) {
    printf("no valid IDstring!\n");
    closesocket(s);
    return 1;
  };
  printf("ID string: %s\n",IDstring);
  if (strncmp(IDstring,myID,12)) {
    printf("Cannot communicate with that fellow. He's too good for me.\n");
    closesocket(s);
    return 1;
  };
  printf("Ok, sounds good.\n");
  do {
#ifdef READLINE
    c=readline("Rcli> ");
    if (c && *c) add_history(c);
#else
    printf("Rcli> ");
    c[1023]=0;
    fgets(c,1023,stdin);
    rq=c; while(*rq && rq-c<1023) { if (*rq=='\n') *rq=0; rq++; };
#endif

    if (!strcmp(c,"q") || !strcmp(c,"quit") || !strcmp(c,"exit")) {
      printf("ok, you got enough, right? leaving.\n");
      break;
    };
    if (!strcmp(c,"shutdown")) {
      printf("performing shutdown.\n");
      doCmd(s,CMD_shutdown,0,0);
    } else if (*c) {
      rq=(char*)malloc(strlen(c)+16);
      *((int*)rq)=itop(SET_PAR(DT_STRING,strlen(c)+2));
      strcpy(rq+4,c);
      strcat(rq+4,"\n");
      printf("sending request\n");
      doCmd(s,CMD_eval,strlen(c)+6,rq);
      free(rq);
    }
  } while (1);

  closesocket(s);
  return 0;
};
